@@ -12,7 +12,14 @@ use constant MORNING   => '08';
 use constant AFTERNOON => '14';
 use constant EVENING   => '20';
 
-our $VERSION = '1.42';
+our $VERSION = '1.47';
+
+my $multiply_by = sub
+{
+    my ($value, $opts) = @_;
+    return $value * $opts->{multiply_by} if exists $opts->{multiply_by};
+    return $value;
+};
 
 sub _no_op
 {
@@ -26,7 +33,7 @@ sub _ago_variant
     my $self = shift;
     $self->_register_trace;
     my $opts = pop;
-    $self->_subtract($opts->{unit} => shift);
+    $self->_subtract($opts->{unit} => $multiply_by->(shift, $opts));
 }
 
 sub _now_variant
@@ -38,7 +45,7 @@ sub _now_variant
     $self->_add_or_subtract({
         when  => $when,
         unit  => $opts->{unit},
-        value => $value,
+        value => $multiply_by->($value, $opts),
     });
 }
 
@@ -88,10 +95,11 @@ sub _hourtime_variant
     my $hours = $opts->{hours} || 0;
     if ($self->_valid_time(hour => $hours)) {
         $self->_set(hour => $hours);
+        $self->{datetime}->set(minute => 0, second => 0, nanosecond => 0);
         $self->_add_or_subtract({
             when  => $when,
-            unit  => 'hour',
-            value => $value,
+            unit  => $opts->{unit},
+            value => $multiply_by->($value, $opts),
         });
     }
 }
@@ -175,7 +183,7 @@ sub _unit_variant
     $self->_add_or_subtract({
         when  => $when,
         unit  => $opts->{unit},
-        value => 1,
+        value => $multiply_by->(1, $opts),
     });
 }
 
@@ -196,7 +204,7 @@ sub _in_count_variant
     my $self = shift;
     $self->_register_trace;
     my $opts = pop;
-    $self->_add_or_subtract($opts->{unit} => shift);
+    $self->_add_or_subtract($opts->{unit} => $multiply_by->(shift, $opts));
 }
 
 sub _month_variant
@@ -246,20 +254,19 @@ sub _count_weekday_variant_month
     }
 }
 
-sub _daytime_hours_variant
+sub _daytime_unit_variant
 {
     my $self = shift;
     $self->_register_trace;
     my $opts = pop;
-    my ($hours, $when, $days) = @_;
-    my %values = (
-        -1 => { day => ($days - 1), hours => (24 - $hours) },
-         1 => { day => $days,       hours => (0  + $hours) },
-    );
-    if ($self->_valid_time(hour => $values{$when}->{hours})) {
-        $self->_add(day => $values{$when}->{day});
-        $self->_set(hour => $values{$when}->{hours});
-    }
+    my ($value, $when, $days) = @_;
+    $self->_add(day => $days);
+    $self->{datetime}->set(hour => 0, minute => 0, second => 0, nanosecond => 0);
+    $self->_add_or_subtract({
+        when  => $when,
+        unit  => $opts->{unit},
+        value => $multiply_by->($value, $opts),
+    });
 }
 
 # wrapper for <time> AM/PM
@@ -283,8 +290,9 @@ sub _at_time
     my $self = shift;
     my $opts = pop;
     my ($time) = @_;
-    my @units = qw(hour minute second);
-    my %values = map { shift @units => $_ } split /:/, $time;
+    my @units = qw(hour minute second nanosecond);
+    my %values = map { shift @units => $_ } split /[:\.]/, $time;
+    $values{nanosecond} *= 1_000_000 if exists $values{nanosecond}; # milli to nano
     if ($self->_valid_time(%values)) {
         $self->_set(%values);
     }
@@ -410,6 +418,18 @@ sub _variant_quarter
     });
 }
 
+sub _begin_end_month
+{
+    my $self = shift;
+    $self->_register_trace;
+    my $opts = pop;
+    my ($day) = @_;
+    unless (defined $day) {
+        $day = $self->_Days_in_Month($self->{datetime}->year, $self->{datetime}->month);
+    }
+    $self->_set(day => $day);
+}
+
 1;
 __END__
 
